/*
 * libGGI graphics driver - dduffey@ksu.edu
 *
 * Fri Feb 13 21:33:00 CST 1998
 * David Duffey
 * keyboard input works
 *
 * Fri Feb 13 15:50:00 CST 1998
 * David Duffey
 * linear frame buffer works, palette works (8bpp)
 *
 * Fri Feb 13 14:04:20 CST 1998
 * David Duffey
 * Time of creation. (excuse for not doing HW)
 */

/*
 * TODO
 *
 *  DirectBuffer
 *  Mouse
 *  write some Exit code :)
 *  keyboard repeat problem?
 *  buffering?
 *  modes besides BM_LINEAR?
 *  different depths
 *
 */

#include "dlinux.h"

#ifdef HAVE_GGI
#include <stdlib.h>
#include <stdio.h>
#include <ggi/libggi.h>
#include "error.h"
#include "gr.h"
#include "mono.h"
#include "key.h"
#include "timer.h"
#include "dggi.h"

int granularity=0;
int xsize=0,ysize=0,bm_type=-1,depth=0;
int running=0;

// ggi stuff
ggi_mode dggi_mode;
ggi_visual_t dggi_vis;
ggi_color dggi_pal[256];
ggi_info *dggi_info;


typedef struct {
  int width;
  int height;
} dggi_vidmode;

#define DGGIMODES 10
const dggi_vidmode dggi_vidmode_table[DGGIMODES] = {
{320,200},
{320,200},
{320,240},
{360,200},
{360,240},
{376,282},
{320,400},
{320,480},
{360,400},
{360,480}, // FIXME more modes, different depths?
};

int dggi_set_mode(int mode) {
  int err;
  // set mode
  depth=GT_8BIT;
  if (mode<DGGIMODES) {
    xsize=dggi_vidmode_table[mode].width;
    ysize=dggi_vidmode_table[mode].height;
  }
  else {
    xsize=320;
    ysize=200;
  }
  ggiInit();
  dggi_vis=ggiOpen(NULL);
  err=ggiSetGraphMode(dggi_vis,xsize,ysize,xsize,ysize,depth);
  // read it back
  // FIXME depth support
  ggiGetMode(dggi_vis,&dggi_mode);
  dggi_info=ggiGetInfo(dggi_vis);
  xsize=dggi_mode.visible.x; ysize=dggi_mode.visible.y;

  if(err) {
    fprintf(stderr,"\nCan't set mode\n");
    return -1;
  }

//FIXME
  bm_type=BM_LINEAR;
  
  granularity=32;
  running=1;
  return 0;
}

void dggi_set_palette(char *pal) {
  register int i;
  if(!running) return;
  for(i=0;i<256;i++) {
    dggi_pal[i].r=(*pal++)<<10;
    dggi_pal[i].g=(*pal++)<<10;
    dggi_pal[i].b=(*pal++)<<10;
  }
  ggiSetPaletteVec(dggi_vis,0,256,dggi_pal);
  return;
}

void dggi_get_palette(char *pal) {
  register int i;
  if (!running) return;
  for(i=0;i<265;i++) {
    *pal++=(dggi_pal[i].r>>10);
    *pal++=(dggi_pal[i].g>>10);
    *pal++=(dggi_pal[i].b>>10);
  }
  return;
}

void dggi_set_drawing_page(int page) {
  return; // FIXME add later
  if (!running) return;
  return;
}

void dggi_set_viewing_page(int page) {
  return; // FIXME
  if(!running) return;
  return;
}

void dggi_bank(int bank) {
  if(!running) return; // FIXME
  return;
}

void dggi_modex_plane(int plane) {
  if(!running) return; // FIXME
  return;
}

void dggi_update() {
  ggiFlush(dggi_vis);
  return;
}

int dggi_get_gran() { // FIXME
  return granularity;
}

unsigned char *dggi_get_buffer() {
  if(!running) return NULL;
  return dggi_info->fb.linear; // FIXME, is a const, use DirectBuffer
}

int dggi_bm_type() {
  return bm_type;
}

int dggi_keyboard_init() {
  return 0;
}

void dggi_keyboard_update() {
  int kcode,ksym,temp;
  ggi_event ev;
  int evmask=emKeyPress|emKeyRelease;
  struct timeval t={0,0};
  while (ggiEventPoll(dggi_vis,evmask,&t)) {
    ggiEventRead(dggi_vis,&ev,evmask);
    // FIXME repeat problem
    // FIXME kcode->descent
    switch (ev.any.type) {
      case evKeyPress:
             kcode=dggi_keycode_to_descent[ev.key.code];
             ksym=ev.key.sym;

             keyd_last_pressed=kcode;
             keyd_time_when_last_pressed=timer_get_fixed_secondsX();
             if (!keyd_pressed[kcode]) {
               key_data.NumDowns[kcode]++;
               keyd_pressed[kcode]=1;
               key_data.TimeKeyWentDown[kcode]=timer_get_fixed_secondsX();
             }
             kcode=kcode 
                   |(keyd_pressed[KEY_LSHIFT]
                     ||keyd_pressed[KEY_RSHIFT]?KEY_SHIFTED:0)
                   |(keyd_pressed[KEY_LALT]
                     ||keyd_pressed[KEY_RALT]?KEY_ALTED:0)
                   |(keyd_pressed[KEY_LCTRL]
                     ||keyd_pressed[KEY_RCTRL]?KEY_CTRLED:0);
             if ((temp=key_data.keytail+1)>=KEY_BUFFER_SIZE) temp=0;
             if (temp!=key_data.keyhead) {
               key_data.keybuffer[key_data.keytail]=kcode;
               key_data.time_pressed[key_data.keytail]= \
                 keyd_time_when_last_pressed;
               key_data.keytail=temp;
             } 

             break;
      case evKeyRelease:
             kcode=dggi_keycode_to_descent[ev.key.code];
             ksym=ev.key.sym;

             keyd_last_released=kcode;
             keyd_pressed[kcode]=0;
             key_data.NumUps[kcode]++;
             key_data.TimeKeyHeldDown[kcode]+= \
               timer_get_fixed_secondsX() \
               - key_data.TimeKeyWentDown[kcode];

             break;
    }
  }
  return;
}

#endif // HAVE_GGI
